@@ -30,6 +30,7 @@ import Plugins.Monitors
 import Plugins.Date
 import Plugins.PipeReader
 import Plugins.BufferedPipeReader
+import Plugins.MarqueePipeReader
 import Plugins.CommandReader
 import Plugins.StdinReader
 import Plugins.XMonadLog
@@ -53,6 +54,7 @@ data Config =
            , position :: XPosition  -- ^ Top Bottom or Static
            , border :: Border       -- ^ NoBorder TopB BottomB or FullB
            , borderColor :: String  -- ^ Border color
+           , borderWidth :: Int     -- ^ Border width
            , hideOnStart :: Bool    -- ^ Hide (Unmap) the window on
                                     --   initialization
            , allDesktops :: Bool    -- ^ Tell the WM to map to all desktops
@@ -65,6 +67,7 @@ data Config =
                                     --   window stack on initialization
            , persistent :: Bool     -- ^ Whether automatic hiding should
                                     --   be enabled or disabled
+           , iconRoot :: FilePath   -- ^ Root folder for icons
            , commands :: [Runnable] -- ^ For setting the command,
                                     --   the command arguments
                                     --   and refresh rate for the programs
@@ -109,12 +112,14 @@ defaultConfig =
            , position = Top
            , border = NoBorder
            , borderColor = "#BFBFBF"
+           , borderWidth = 1
            , hideOnStart = False
            , lowerOnStart = True
            , persistent = False
            , allDesktops = True
            , overrideRedirect = True
            , pickBroadest = False
+           , iconRoot = "."
            , commands = [ Run $ Date "%a %b %_d %Y * %H:%M:%S" "theDate" 10
                         , Run StdinReader]
            , sepChar = "%"
@@ -136,6 +141,6 @@ infixr :*:
 -- this function's type signature.
 runnableTypes :: Command :*: Monitors :*: Date :*: PipeReader :*: BufferedPipeReader :*: CommandReader :*: StdinReader :*: XMonadLog :*: EWMH :*: Kbd :*: Locks :*:
                  Mail :*: MBox :*:
-                 DateZone :*:
+                 DateZone :*: MarqueePipeReader :*:
                  ()
 runnableTypes = undefined
