@@ -40,6 +40,8 @@ module Plugins.Monitors.Common (
                        , parseTemplate'
                        -- ** String Manipulation
                        -- $strings
+                       , IconPattern
+                       , parseIconPattern
                        , padString
                        , showWithPadding
                        , showWithColors
@@ -48,8 +50,10 @@ module Plugins.Monitors.Common (
                        , showPercentsWithColors
                        , showPercentBar
                        , showVerticalBar
+                       , showIconPattern
                        , showLogBar
                        , showLogVBar
+                       , showLogIconPattern
                        , showWithUnits
                        , takeDigits
                        , showDigits
@@ -60,6 +64,7 @@ module Plugins.Monitors.Common (
                        ) where
 
 
+import Control.Applicative ((<$>))
 import Control.Monad.Reader
 import qualified Data.ByteString.Lazy.Char8 as B
 import Data.IORef
@@ -112,7 +117,7 @@ mods s m =
 
 setConfigValue :: a -> Selector a -> Monitor ()
 setConfigValue v s =
-       mods s (\_ -> v)
+       mods s (const v)
 
 getConfigValue :: Selector a -> Monitor a
 getConfigValue = sel
@@ -342,11 +347,23 @@ combine m ((s,ts,ss):xs) =
     do next <- combine m xs
        str <- case Map.lookup ts m of
          Nothing -> return $ "<" ++ ts ++ ">"
-         Just  r -> let f "" = r; f n = n; in fmap f $ parseTemplate' r m
+         Just  r -> let f "" = r; f n = n; in f <$> parseTemplate' r m
        return $ s ++ str ++ ss ++ next
 
 -- $strings
 
+type IconPattern = Int -> String
+
+parseIconPattern :: String -> IconPattern
+parseIconPattern path =
+    let spl = splitOnPercent path
+    in \i -> concat $ intersperse (show i) spl
+  where splitOnPercent [] = [[]]
+        splitOnPercent ('%':'%':xs) = [] : splitOnPercent xs
+        splitOnPercent (x:xs) =
+            let rest = splitOnPercent xs
+            in (x : head rest) : tail rest
+
 type Pos = (Int, Int)
 
 takeDigits :: Int -> Float -> Float
@@ -452,6 +469,15 @@ showPercentBar v x = do
   s <- colorizeString v (take len $ cycle bf)
   return $ s ++ take (bw - len) (cycle bb)
 
+showIconPattern :: Maybe IconPattern -> Float -> Monitor String
+showIconPattern Nothing _ = return ""
+showIconPattern (Just str) x = return $ str $ convert $ 100 * x
+  where convert val
+          | t <= 0 = 0
+          | t > 8 = 8
+          | otherwise = t
+          where t = round val `div` 12
+
 showVerticalBar :: Float -> Float -> Monitor String
 showVerticalBar v x = colorizeString v [convert $ 100 * x]
   where convert :: Float -> Char
@@ -459,18 +485,20 @@ showVerticalBar v x = colorizeString v [convert $ 100 * x]
           | t <= 9600 = ' '
           | t > 9608 = chr 9608
           | otherwise = chr t
-          where t = 9600 + ((round val) `div` 12)
+          where t = 9600 + (round val `div` 12)
 
 showLogBar :: Float -> Float -> Monitor String
-showLogBar f v = do
-  h <- fromIntegral `fmap` getConfigValue high
-  l <- fromIntegral `fmap` getConfigValue low
-  bw <- fromIntegral `fmap` getConfigValue barWidth
-  let [ll, hh] = sort [l, h]
-      choose x | x == 0.0 = 0
-               | x <= ll = 1 / bw
-               | otherwise = f + logBase 2 (x / hh) / bw
-  showPercentBar v $ choose v
+showLogBar f v = 
+  let intConfig c = fromIntegral `fmap` getConfigValue c
+  in do
+    h <- intConfig high
+    l <- intConfig low
+    bw <- intConfig barWidth
+    let [ll, hh] = sort [l, h]
+        choose x | x == 0.0 = 0
+                 | x <= ll = 1 / bw
+                 | otherwise = f + logBase 2 (x / hh) / bw
+    showPercentBar v $ choose v
 
 showLogVBar :: Float -> Float -> Monitor String
 showLogVBar f v = do
@@ -482,3 +510,14 @@ showLogVBar f v = do
                | x <= ll = 1 / bw
                | otherwise = f + logBase 2 (x / hh) / bw
   showVerticalBar v $ choose v
+
+showLogIconPattern :: Maybe IconPattern -> Float -> Float -> Monitor String
+showLogIconPattern str f v = do
+  h <- fromIntegral `fmap` getConfigValue high
+  l <- fromIntegral `fmap` getConfigValue low
+  bw <- fromIntegral `fmap` getConfigValue barWidth
+  let [ll, hh] = sort [l, h]
+      choose x | x == 0.0 = 0
+               | x <= ll = 1 / bw
+               | otherwise = f + logBase 2 (x / hh) / bw
+  showIconPattern str $ choose v
