@@ -24,8 +24,8 @@ import Plugins.Monitors.CoreCommon
 cpuFreqConfig :: IO MConfig
 cpuFreqConfig = mkMConfig
        "Freq: <cpu0>" -- template
-       (zipWith (++) (repeat "cpu") (map show [0 :: Int ..])) -- available
-                                                              -- replacements
+       (map ((++) "cpu" . show) [0 :: Int ..]) -- available
+                                             -- replacements
 
 -- |
 -- Function retrieves monitor string holding the cpu frequency (or frequencies)
@@ -33,7 +33,7 @@ runCpuFreq :: [String] -> Monitor String
 runCpuFreq _ = do
   let path = ["/sys/devices/system/cpu/cpu", "/cpufreq/scaling_cur_freq"]
       divisor = 1e6 :: Double
-      fmt x | x < 1 = (show (round (x * 1000) :: Integer)) ++ "MHz"
-            | otherwise = (show x) ++ "GHz"
+      fmt x | x < 1 = show (round (x * 1000) :: Integer) ++ "MHz"
+            | otherwise = show x ++ "GHz"
   failureMessage <- getConfigValue naString
   checkedDataRetrieval failureMessage [path] Nothing (/divisor) fmt
