@@ -24,7 +24,7 @@ import System.Console.GetOpt
 
 volumeConfig :: IO MConfig
 volumeConfig = mkMConfig "Vol: <volume>% <status>"
-                         ["volume", "volumebar", "volumevbar", "dB","status"]
+                         ["volume", "volumebar", "volumevbar", "dB","status", "volumeipat"]
 
 
 data VolumeOpts = VolumeOpts
@@ -34,6 +34,7 @@ data VolumeOpts = VolumeOpts
     , offColor :: Maybe String
     , highDbThresh :: Float
     , lowDbThresh :: Float
+    , volumeIconPattern :: Maybe IconPattern
     }
 
 defaultOpts :: VolumeOpts
@@ -44,6 +45,7 @@ defaultOpts = VolumeOpts
     , offColor = Just "red"
     , highDbThresh = -5.0
     , lowDbThresh = -30.0
+    , volumeIconPattern = Nothing
     }
 
 options :: [OptDescr (VolumeOpts -> VolumeOpts)]
@@ -54,6 +56,8 @@ options =
     , Option "" ["highd"] (ReqArg (\x o -> o { highDbThresh = read x }) "") ""
     , Option "C" ["onc"] (ReqArg (\x o -> o { onColor = Just x }) "") ""
     , Option "c" ["offc"] (ReqArg (\x o -> o { offColor = Just x }) "") ""
+    , Option "" ["volume-icon-pattern"] (ReqArg (\x o ->
+       o { volumeIconPattern = Just $ parseIconPattern x }) "") ""
     ]
 
 parseOpts :: [String] -> IO VolumeOpts
@@ -80,6 +84,10 @@ formatVolVBar :: Integer -> Integer -> Integer -> Monitor String
 formatVolVBar lo hi v =
     showVerticalBar (100 * x) x where x = percent v lo hi
 
+formatVolDStr :: Maybe IconPattern -> Integer -> Integer -> Integer -> Monitor String
+formatVolDStr ipat lo hi v =
+    showIconPattern ipat $ percent v lo hi
+
 switchHelper :: VolumeOpts
              -> (VolumeOpts -> Maybe String)
              -> (VolumeOpts -> String)
@@ -126,7 +134,8 @@ runVolume mixerName controlName argv = do
     v <- liftMonitor $ liftM3 formatVolVBar lo hi val
     d <- getFormatDB opts db
     s <- getFormatSwitch opts sw
-    parseTemplate [p, b, v, d, s]
+    ipat <- liftMonitor $ liftM3 (formatVolDStr $ volumeIconPattern opts) lo hi val
+    parseTemplate [p, b, v, d, s, ipat]
 
   where
 
