@@ -19,7 +19,7 @@ import Plugins
 import System.Posix.Files
 import Control.Concurrent(threadDelay)
 import Control.Exception
-import Control.Monad(when)
+import Control.Monad(forever, unless)
 
 data PipeReader = PipeReader String String
     deriving (Read, Show)
@@ -28,21 +28,18 @@ instance Exec PipeReader where
     alias (PipeReader _ a)    = a
     start (PipeReader p _) cb = do
         let (def, pipe) = split ':' p
-        when (not $ null def) (cb def)
+        unless (null def) (cb def)
         checkPipe pipe
         h <- openFile pipe ReadWriteMode
         forever (hGetLineSafe h >>= cb)
       where
-        forever a = a >> forever a
-        split c xs | c `elem` xs = let (pre, post) = span ((/=) c) xs
-                                   in (pre, dropWhile ((==) c) post)
+        split c xs | c `elem` xs = let (pre, post) = span (c /=) xs
+                                   in (pre, dropWhile (c ==) post)
                    | otherwise   = ([], xs)
 
 checkPipe :: FilePath -> IO ()
-checkPipe file = do
+checkPipe file =
     handle (\(SomeException _) -> waitForPipe) $ do
-    status <- getFileStatus file
-    if isNamedPipe status
-      then return ()
-      else waitForPipe
+        status <- getFileStatus file
+        unless (isNamedPipe status) waitForPipe
     where waitForPipe = threadDelay 1000 >> checkPipe file
